
# --- libs ---
import pygame as pg
import sys
import fish
import os


# --- basescreen ---
class BaseScreen:
	def __init__(self, draw, update, inputreg=None):
		self.draw = draw
		self.update = update
		self.inputreg = inputreg


	def run(self, win, clock):

		run = True #not equal to self.run()
		continue_ = True
		while run:

			self.draw(win)
			run, options = self.update()

			for evt in pg.event.get():
				if evt.type == pg.QUIT:
					run, continue_ = False, False

				if evt.type == pg.KEYDOWN:
					if self.inputreg:
						self.inputreg(evt)

			clock.tick(60)
			pg.display.update()

		return continue_, options


# --- error ---
class Error:
	def __init__(self, error):
		self.error = error
		self.l1 = fish.Label('ERROR MESSAGE', 10, 20, color=(0,0,0), size=40)
		self.l2 = fish.Label('', 10, 60, color=(0,0,0))
		self.l2.set_text(error)


	def run(self):

		win = pg.display.set_mode((400,400))
		pg.display.set_caption('ERROR')

		# from stackoverflow
		try:
			exc_type, exc_obj, exc_tb = sys.exc_info()
			fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
			self.l2.set_text([self.error,exc_type, fname + ', line ' + str(exc_tb.tb_lineno)])
		except:
			pass


		while True:
			for evt in pg.event.get():
				if evt.type == pg.QUIT:
					sys.exit()

			win.fill((255,255,255))
			self.l1.draw(win)
			self.l2.draw(win)
			pg.display.update()


# --- pop measge ---
class PopMsg:
	def __init__(self, text, time, id_type=None, size=20, color=(0,0,0)):
		text = str(text)

		"""
		in main loop
	
		for i in popmesages:
			popmessages = i.show(win, popmessages)
		
		always have this running.

		when a new pop message shall appear then
		simply add it onto the list.

		"""
		info = pg.display.Info()
		w,h = info.current_w, info.current_h

		self.lb = fish.gui.Label(str(text), w/2-len(text)*size/2, h*0.4, size=size, color=color)
		self.time = time
		self.clock = 0

		# id type used for changing of pop of message i.e you have to zoom.
		# and stacking pop up messages is a bad idea.
		self.id_type = id_type


	def show(self, win, l=None):
		
		if self.time > self.clock:
			self.lb.draw(win)
		self.clock += 1

		if len(l) > 1:
			for i in l:
				if i.id_type == self.id_type: #i == self
					l.pop(l.index(i))

		return l






# --- libs --- 
from random import randint
import pygame as pg

# --- class ---
class Button:
	def __init__(self, x, y, img=None, size=None, color=(0,0,150),highlight=None,base=None, drawbackground=True):
		
		# - ui -
		self.x = x
		self.y = y
		self.color = color
		self.ogColor = tuple(color)

		self.tp = 'not declared'
		
		if img:
			self.tp = 'image'
			self.w, self.h = img.get_size()
			self.img = img

		elif size:
			self.tp = 'size'
			self.w = size[0]
			self.h = size[1]

		else:
			print('Please enter a size or an image')

		self.id = str(randint(100,999))

		# - mechanics -
		self.press = False
		self.highlight = highlight
		if base:
			self.baseX = int(x)
			self.baseY = int(y)
		self.on = True
		self.drawbackground = drawbackground
		self.hover = False


	def draw(self, win, correction=None):
		
		if correction:
			x = self.x - correction[0]
			y = self.y - correction[1]

		else:
			x = self.x
			y = self.y


		if self.tp == 'image':
			win.blit(self.img, (x, y))
		
		elif self.tp == 'size':
			if self.drawbackground:
				pg.draw.rect(win, self.color, [x, y, self.w, self.h])
			else:
				if self.color != self.ogColor:
					pg.draw.rect(win, self.color, [x, y, self.w, self.h])


	
	def get_pressed(self):

		if self.tp != 'not declared' and self.on:
			
			mouse_pos = pg.mouse.get_pos()

			if mouse_pos[0] in range(round(self.x), round(self.x + self.w)) and mouse_pos[1] in range(round(self.y), round(self.y + self.h)):
		
				self.hover = True
				if self.highlight:
					self.color = self.highlight

				if self.press and pg.mouse.get_pressed()[0] == 0:
					self.press = False
					return True

				if pg.mouse.get_pressed()[0] == 1:
					self.press = True

			else:
				self.hover = False
				self.press = False
				self.color = self.ogColor

	def get_size(self):
		return (self.w, self.h)

	def get_info(self):
		return {'x':self.x, 'y':self.y, 'w':self.w, 'h':self.h, 'id':self.id, 'highlight':self.highlight, 'size':self.get_size()}




# --- libs --- 
import pygame as pg
import fish
import os


# --- class ---
class Gallery:
	def __init__(self, x, y, w, h, color=(0,0,0),ES=80):
		self.ES = ES
		
		self.x = x
		self.y = y
		self.w = w
		self.h = h
		self.color = color

		self.elements = {1:[]} #add dict to it, with surface and button
		self.freepos = [0,0]
		self.c = 1

		# display surfaces on buttons
		# in a container

		self.current_screen = 1
		self.avaliable_screen = 1

		leftimg = pg.image.load('left.png')
		self.left = fish.gui.Button(self.x+self.w*0.75,self.y-25,img=leftimg)
		self.left_highlight = fish.gui.Button(self.x+self.w*0.75,self.y-25,size=leftimg.get_size(), color=(255,255,255), highlight=(200,200,200))

		rightimg = pg.image.load('right.png')
		self.right = fish.gui.Button(self.x+self.w*0.92,self.y-25,img=rightimg)
		self.right_highlight = fish.gui.Button(self.x+self.w*0.92,self.y-25,size=rightimg.get_size(), color=(255,255,255), highlight=(200,200,200))




	def _resize(self, img):
		return pg.transform.scale(img, (int(self.ES*0.5), int(self.ES*0.5)))


	def _newpos(self):

		self.freepos[0] = self.freepos[0] + 1

		if self.freepos[0] * self.ES > self.w - self.ES:
			self.freepos[0] = 0
			self.freepos[1] = self.freepos[1] + 1


	def _maketext(self, text, x, y):
		
		mx = 8

		label = fish.gui.Label('<3', x+self.ES*0.05, y+self.ES*0.6, color=self.color, size=int(self.ES*0.15))

		tx = []

		if len(text) > mx:
			tx = [text[:mx]]
			if len(text[mx:]) > mx:
				tx.append(text[mx:mx*2])
			else:
				tx.append(text[mx:])

		else:
			tx = text

		label.set_text(tx)

		return label


	def add(self, l):
		
		# 50, 50 for every element

		for e in l:

			if self.freepos[1] >= round(self.h/self.ES):
				self.c += 1
				self.elements[self.c] = []
				self.freepos[1] = 0
				self.avaliable_screen += 1

			d = {}
			x = self.freepos[0]*self.ES+self.x+3
			y =self.freepos[1]*self.ES+self.y+3
			d['fullname'] = str(e[0])
			d['name'] = self._maketext(e[0], x,y)
			d['image'] = self._resize(e[1])
			d['button'] = fish.gui.Button(x, y, size=(self.ES,self.ES), color=(255,255,255), highlight=(200,200,200))
			d['pos'] = (x,y)
			
			self.elements[self.c].append(d)

			self._newpos()


	def draw(self, win):
		
		for elm in self.elements[self.current_screen]:
			
			elm['button'].draw(win)
			win.blit(elm['image'], (elm['pos'][0]+self.ES*0.05, elm['pos'][1]+self.ES*0.1))
			elm['name'].draw(win)

		fish.sharprect(win, self.color, [self.x, self.y, self.w, self.h], 5)
		
		self.left_highlight.draw(win)
		self.right_highlight.draw(win)
		self.left.draw(win)
		self.right.draw(win)


	def update(self):

		for elm in self.elements[self.current_screen]:
			if elm['button'].get_pressed():
				return elm

		if self.left.get_pressed():
			self.current_screen -= 1
			if self.current_screen < 1:
				self.current_screen = 1

		if self.right.get_pressed():
			self.current_screen += 1
			if self.current_screen > self.avaliable_screen:
				self.current_screen -= 1

		# highlight
		self.left_highlight.get_pressed()
		self.right_highlight.get_pressed()
		return 0



# --- libs ---
from PIL import Image
import os
import pygame as pg


# --- rotate image ---
def rotate(name, angle_range):

	r_images = []
	image = Image.open(name)

	for i in range(angle_range[0], angle_range[1]):
		
		image_copy = image.copy()
		image_copy = image_copy.rotate(i)

		mode = image_copy.mode
		size = image_copy.size
		data = image_copy.tobytes()

		r_images.append(pg.image.fromstring(data, size, mode))
	
	return r_images


# --- functions ---
# only works for pygame surfaces
def change_color(surface, color):
   
    """Fill all pixels of the surface with color, preserve transparency."""
    w, h = surface.get_size()
    r, g, b, _ = color
    for x in range(w):
        for y in range(h):
            a = surface.get_at((x, y))[3]
            surface.set_at((x, y), pg.Color(r, g, b, a))

# credit to stackoverflow user 'skrx'




# helper class to handle animations
# animationhandler


# --- libs ---
import pygame as pg
import fish
import os, math
from random import randint


# --- class --- 
class Animation:
	def __init__(self, name, total, tick):
	
		self.images = []

		for i in range(total):
			self.images.append(pg.image.load(os.path.join(os.getcwd(), "assets", name, name+str(i)+".png")).convert_alpha())

		self.pic = self.images[0]
		self.picCount = 0

		self.timer = randint(0,50)
		self.tick = tick
		self.name = name

	def draw(self, win, place):
		if self.timer % self.tick == 0:
			self.picCount += 1
			if self.picCount > len(self.images)-1:
				self.picCount = 0
			self.pic = self.images[self.picCount]

		
		self.timer += 1
		win.blit(self.pic, place)



# needs polishing
# old file
# please update, future me

# --- libs ---
import pygame as pg
import fish
import os
from random import randint


# --- class --- 
class PersonAnimation:
	def __init__(self):
		
		names = ["graverobber", "steamman", "woodcutter"] #,"woman"]
		self.name = names[randint(0, len(names)-1)]

		path = os.path.join(os.getcwd(), "assets", self.name)
		dist = 48

		self.img_running = []
		self.img_idle = []
		self.img_damage = []
		self.img_jump = []
		self.img_standard = pg.image.load(os.path.join(path, self.name + ".png"))


		for i in range(4):
			self.img_idle.append(self.crop(pg.image.load(os.path.join(path, "_idle.png")), (40,64,-dist*i,0)))
		
		for i in range(6):
			self.img_running.append(self.crop(pg.image.load(os.path.join(path, "_run.png")), (40,64,-dist*i,0)))

		for i in range(1,2):
			self.img_damage.append(self.crop(pg.image.load(os.path.join(path, "_hurt.png")), (40,64,-dist*i,0)))

		l = []
		for i in range(6):
			l.append(self.crop(pg.image.load(os.path.join(path, "_jump.png")), (40,64,-dist*i,0)))

		self.img_jump = [[l[0], l[1], l[2]], [l[3]], [l[4], l[5]] ]


		self.img = list(self.img_running)
		self.img_count = 0
		self.pic = self.img[self.img_count]

		self.timer = 0
		self.tick = 8
		self.state = ""
		self.d_state = ""
		self.place = (0,0)
		self.ori = "left"

	def crop(self, img, size):
		
		cImg = pg.Surface((size[0], size[1]))
		cImg.blit(img, (size[2], size[3]))

		return cImg


	def switch_animation(self, state):
		self.img_count = 0
		
		if state == "idle":
			self.img = list(self.img_idle)

		elif state == "jump":
			self.img = list(self.img_jump[0])

		elif state == "left":
			self.img = list(self.img_running)

		elif state == "right":
			self.img = list(self.img_running)

		else:
			self.img = list(self.img_jump[1])

		
	def animate(self, platform):
		
		# - close to ground -
		if not self.state:

			for i in range(len(platform)):
				if platform[i] - self.place[1] <= 120:
					self.img = self.img_jump[2]


		if self.timer % self.tick == 0:
			self.img_count += 1
		
			if self.img_count >= len(self.img) and self.state != "jump":
				self.img_count = 0
		
		self.timer += 1

		# - desiered state -
		if self.state != self.d_state:
			self.pic = self.img_standard
			self.state = self.d_state
			self.switch_animation(self.state)
		
		
		#print(len(self.img), self.img_count)
		self.pic = self.img[self.img_count]

		# - jump -
		if self.state == "jump" and self.img_count == len(self.img)-1:
			self.timer = 1
			self.img_count == len(self.img)-1
			self.pic = self.img_jump[1][0]


	def draw(self, win, place):
		self.place = place
		if self.ori == "left":
			self.pic = pg.transform.flip(self.pic, True, False)

		win.blit(self.pic, place)